'use client';

import { IStream } from "@/lib/streams";
import NotFound from "app/s/[cuid]/not-found";
import { IVod } from "@/lib/vods";
import Link from "next/link";
import Image from "next/image";
import { LocalizedDate } from "./localized-date";
import { FontAwesomeIcon, FontAwesomeIconProps } from "@fortawesome/react-fontawesome";
import { faCheck, faTriangleExclamation, faCircleInfo, faThumbsUp, IconDefinition } from "@fortawesome/free-solid-svg-icons";
import styles from '@/assets/styles/fp.module.css'
import { Hemisphere, Moon } from "lunarphase-js";
import { useEffect, useState } from "react";


export interface IStreamProps {
    stream: IStream;
}
type Status = 'missing' | 'issue' | 'good';
interface StyleDef {
    heading: string;
    icon: IconDefinition;
    desc1: string;
    desc2: string;
}

function capitalizeFirstLetter(string: string): string {
    return string.charAt(0).toUpperCase() + string.slice(1);
}

export default function StreamPage({ stream }: IStreamProps) {
    if (!stream) return <NotFound></NotFound>
    const displayName = stream.attributes.vtuber.data.attributes.displayName;
    const date = new Date(stream.attributes.date);
    const [hemisphere, setHemisphere] = useState(Hemisphere.NORTHERN);
    const [selectedStatus, setSelectedStatus] = useState<Status>('missing');

    const styleMap: Record<Status, StyleDef> = {
        'missing': {
            heading: 'is-danger',
            icon: faTriangleExclamation,
            desc1: "We don't have a VOD for this stream.",
            desc2: 'Know someone who does?'
        },
        'issue': {
            heading: 'is-warning',
            icon: faCircleInfo,
            desc1: "We have a VOD for this stream, but it's not full quality.",
            desc2: 'Have a better copy?'
        },
        'good': {
            heading: 'is-success',
            icon: faThumbsUp,
            desc1: "We have a VOD for this stream, and we think it's the best quality possible.",
            desc2: "Have one that's even better?"
        }
    };
    const { heading, icon, desc1, desc2 } = styleMap[selectedStatus] || {};

    useEffect(() => {
        const randomHemisphere = (Math.random() < 0.5 ? 0 : 1) ? Hemisphere.NORTHERN : Hemisphere.SOUTHERN;
        setHemisphere(randomHemisphere);
    }, []);

    return (
        <>


            <div className="content">
                <div className="section">
                    <h1 className="title"><LocalizedDate date={date} /> {displayName} Stream Archive</h1>
                </div>

                <div className="section columns is-multiline">
                    <div className="column is-half">
                        <div className="box">
                            <h2 className="title is-3">Details</h2>
                            <div className="columns is-multiline">
                                <div className="column is-full">
                                    <span><b>ISO Date</b>&nbsp;</span><span>{date.toISOString()}</span><br></br>
                                    <span><b>Lunar Phase</b>&nbsp;</span><span>{Moon.lunarPhase(date)} {Moon.lunarPhaseEmoji(date, { hemisphere })}</span>
                                    <br></br>
                                    <select className="mt-5"
                                        value={selectedStatus}
                                        onChange={e => setSelectedStatus(e.target.value as Status)}
                                    >
                                        <option>good</option>
                                        <option>issue</option>
                                        <option>missing</option>
                                    </select>
                                </div>
                            </div>
                        </div>
                    </div>


                    <div className="column is-half">
                        <article className={`message ${heading}`}>
                            <div className="message-header">
                                <span>VOD {capitalizeFirstLetter(selectedStatus)}</span>
                            </div>
                            <div className="message-body has-text-centered">
                                <span className="title is-1"><FontAwesomeIcon icon={icon}></FontAwesomeIcon></span>
                                <p className="mt-3">{desc1}</p>
                                <p className="mt-5">{desc2}<br />
                                <Link href={`/upload?cuid=${stream.attributes.cuid}`}>Upload it here.</Link></p>
                            </div>
                        </article>
                    </div>

                </div>


                <div className="section">
                    <h1 className="title">VODs</h1>
                    <table className="table">
                        <thead>
                            <tr>
                                <th><abbr title="Thumbnail">Thmb</abbr></th>
                                <th>Length</th>
                                <th>Uploader</th>
                                <th>Tags</th>
                                <th><abbr title="Timestamps">TS</abbr></th>
                                <th>Note</th>
                            </tr>
                        </thead>
                        <tbody>
                            {stream.attributes.vods.data.map((vod: IVod) => (
                                <tr>
                                    <th>blah</th>
                                </tr>
                            ))}
                        </tbody>
                    </table>
                </div>

            </div>

        </>
    )
}
