import { getVodForDate, getUrl, getNextVod, getPreviousVod, getLocalizedDate } from '@/lib/vods';
import { IVod } from '@/lib/vods';
import { getDateFromSafeDate } from '@/lib/dates';
import Link from 'next/link';
import { VideoInteractive } from './video-interactive';
import { FontAwesomeIcon } from '@fortawesome/react-fontawesome';
import { faChevronLeft, faChevronRight, faGlobe, faLink } from "@fortawesome/free-solid-svg-icons";
import { notFound } from 'next/navigation';
import { IpfsCid } from './ipfs-cid';

export interface IPageProps {
    safeDate: string;
    slug: string;
}

export function getVodTitle(vod: IVod): string {
    return vod.attributes.title || vod.attributes.announceTitle || `${vod.attributes.vtuber.data.attributes.displayName} ${vod.attributes.date2}`;
}

export function buildMuxUrl(playbackId: string, token: string) {
    return `https://stream.mux.com/${playbackId}.m3u8?token=${token}`
}

export function buildMuxSignedPlaybackId(playbackId: string, token: string) {
    return `${playbackId}?token=${token}`
}

export function buildMuxThumbnailUrl(playbackId: string, token: string) {
    return `https://image.mux.com/${playbackId}/storyboard.vtt?token=${token}`
}


export default async function VodPage({ safeDate, slug }: IPageProps) {
    const date: Date = getDateFromSafeDate(safeDate);
    if (!date) notFound();
    // console.log(`(VodPage) lets get a vod for date:${date.toISOString()}`)
    const vod = await getVodForDate(date);
    const localizedDate = date.toLocaleDateString();

    if (!vod) notFound();
    const previousVod = await getPreviousVod(vod);
    const nextVod = await getNextVod(vod);

    // return (<pre>
    //     <code>{JSON.stringify(previousVod, null, 2)}</code>
    // </pre>)

    return (

        <div className="container">
            <div className="block">
                <div className="box">
                    <VideoInteractive vod={vod} safeDate={safeDate} localizedDate={localizedDate}></VideoInteractive>


                    {(vod.attributes.videoSrcHash || vod.attributes.video240Hash) && (
                        <>
                            <h4 className='subtitle is-4'>
                                <a id="ipfs"></a>
                                <Link href="#ipfs">
                                    <FontAwesomeIcon
                                        icon={faLink}
                                        className="fab fa-link mr-2"
                                    ></FontAwesomeIcon>
                                </Link>
                                <FontAwesomeIcon
                                    icon={faGlobe}
                                    className="fab fa-plus mr-2"
                                ></FontAwesomeIcon>
                                <span>IPFS Content IDs</span>
                            </h4>
                            {vod.attributes.videoSrcHash && (
                                <IpfsCid label="Source" cid={vod.attributes.videoSrcHash}></IpfsCid>
                            )}
                            {vod.attributes.video240Hash && (
                                <IpfsCid label="240p" cid={vod.attributes.video240Hash}></IpfsCid>
                            )}
                        </>
                    )}


                    <nav className="level mt-5">
                        <div className='level-left'>
                            <div className='level-item'>
                                {!!previousVod && (
                                    <Link className='button' href={getUrl(previousVod, slug, previousVod.attributes.date2)}>
                                        <FontAwesomeIcon
                                            icon={faChevronLeft}
                                            className='fas faChevronLeft'
                                        ></FontAwesomeIcon>
                                        <span className="ml-2">Prev VOD {getLocalizedDate(previousVod)}</span>
                                    </Link>
                                )}
                            </div>
                        </div>
                        <div className='level-right'>
                            <div className='level-item'>
                                {!!nextVod && (
                                    <Link className='button' href={getUrl(nextVod, slug, nextVod.attributes.date2)}>
                                        <span className="mr-2">Next VOD {getLocalizedDate(nextVod)}</span>
                                        <FontAwesomeIcon
                                            icon={faChevronRight}
                                            className='fas faChevronRight'
                                        ></FontAwesomeIcon>
                                    </Link>
                                )}
                            </div>
                        </div>
                    </nav>


                </div>
            </div>
        </div>
    );
}
