
import { strapiUrl, siteUrl } from './constants';
import { getSafeDate } from './dates';
import { IVodsResponse } from './vods';
import { IVtuber, IVtuberResponse } from './vtubers';
import qs from 'qs';


export interface IStream {
    id: number;
    attributes: {
        date: string;
        archiveStatus: 'good' | 'issue' | 'missing';
        vods: IVodsResponse;
        cuid: string;
        vtuber: IVtuberResponse;
    }
}

export interface IStreamsResponse {
    data: IStream[];
    meta: {
        pagination: {
            page: number;
            pageSize: number;
            pageCount: number;
            total: number;
        }
    }
}


const fetchStreamsOptions = {
    next: {
        tags: ['streams']
    }
}


export async function getStreamByCuid(cuid: string): Promise<IStream> {
    const query = qs.stringify({
        filters: {
            cuid: {
                $eq: cuid
            }
        },
        pagination: {
            limit: 1
        },
        populate: {
            vtuber: {
                fields: ['slug', 'displayName', 'image', 'imageBlur'],
            },
            vods: '*'
        }
    });
    const res = await fetch(`${strapiUrl}/api/streams?${query}`);
    const json = await res.json();
    return json.data[0];
}

export function getUrl(stream: IStream, slug: string, date: string): string {
    return `${siteUrl}/vt/${slug}/stream/${getSafeDate(date)}`
}


export function getPaginatedUrl(): (slug: string, pageNumber: number) => string {
    return (slug: string, pageNumber: number) => {
        return `${siteUrl}/vt/${slug}/streams/${pageNumber}`
    }
}



export function getLocalizedDate(stream: IStream): string {
    return new Date(stream.attributes.date).toLocaleDateString()
}




export async function getStreams(id: number): Promise<IStream> {
    const allStreams: IStream[] = [];
    let currentPage = 1;

    while (true) {
        const query = qs.stringify({
            filters: {
                id: {
                    $eq: id
                }
            },
            pagination: {
                pageSize: 100,
                page: currentPage,
            },
        });
        const response = await fetch(`${strapiUrl}/api/vods?${query}`, fetchStreamsOptions);
        const responseData = await response.json();

        if (!responseData.data || responseData.data.length === 0) {
            // No more data to fetch
            break;
        }

        allStreams.push(...responseData.data);
        currentPage++;
    }

    const query = qs.stringify(
        {
            filters: {
                id: {
                    $eq: id
                }
            }
        }
    )
    const d = await fetch(`${strapiUrl}/api/streams?${query}`, fetchStreamsOptions)
        .then((res) => res.json())
    return d
}




export async function getAllStreams(): Promise<IStream[]> {
    const pageSize = 100; // Adjust this value as needed
    const sortDesc = true; // Adjust the sorting direction as needed

    const allstreams: IStream[] = [];
    let currentPage = 1;

    while (true) {
        const query = qs.stringify({
            populate: {
                vtuber: {
                    fields: ['slug', 'displayName', 'image', 'imageBlur'],
                },
                muxAsset: {
                    fields: ['playbackId', 'assetId'],
                },
                thumbnail: {
                    fields: ['cdnUrl', 'url'],
                },
                tagstreamRelations: {
                    fields: ['tag'],
                    populate: ['tag'],
                },
                videoSrcB2: {
                    fields: ['url', 'key', 'uploadId', 'cdnUrl'],
                },
            },
            sort: {
                date: sortDesc ? 'desc' : 'asc',
            },
            pagination: {
                pageSize,
                page: currentPage,
            },
        });
        const response = await fetch(`${strapiUrl}/api/streams?${query}`, fetchStreamsOptions);
        const responseData = await response.json();

        if (!responseData.data || responseData.data.length === 0) {
            // No more data to fetch
            break;
        }

        allstreams.push(...responseData.data);
        currentPage++;
    }

    return allstreams;
}

export async function getStreamForVtuber(vtuberId: number, safeDate: string): Promise<IStream> {
    const query = qs.stringify({
        populate: {
            vods: {
                fields: [
                    'id',
                    'date'
                ]
            },
            tweet: {
                fields: [
                    'id'
                ]
            }
        }
    });

    const response = await fetch(`${strapiUrl}/api/streams?${query}`, fetchStreamsOptions);

    if (response.status !== 200) throw new Error('network fetch error while attempting to getStreamForVtuber');

    const responseData = await response.json();
    return responseData;
}

export async function getAllStreamsForVtuber(vtuberId: number, page: number = 1, pageSize: number = 25, sortDesc = true): Promise<IStream[]> {
    const maxRetries = 3;

    let retries = 0;
    let allStreams: IStream[] = [];
    let currentPage = 1;

    while (retries < maxRetries) {
        try {
            const query = qs.stringify({
                populate: {
                    vods: {
                        fields: [
                            'id',
                        ]
                    }
                },
                sort: {
                    date: sortDesc ? 'desc' : 'asc',
                },
                pagination: {
                    pageSize,
                    page: currentPage,
                },
            });

            const response = await fetch(`${strapiUrl}/api/streams?${query}`, fetchStreamsOptions);

            if (response.status !== 200) {
                // If the response status is not 200 (OK), consider it a network failure
                retries++;
                continue;
            }

            const responseData = await response.json();

            if (!responseData.data || responseData.data.length === 0) {
                // No more data to fetch
                break;
            }

            allStreams.push(...responseData.data);
            currentPage++;
        } catch (error) {
            // Network failure or other error occurred
            retries++;
        }
    }

    if (retries === maxRetries) {
        throw new Error(`Failed to fetch streams after ${maxRetries} retries.`);
    }

    return allStreams;
}

export async function getStreamsForVtuber(vtuberId: number, page: number = 1, pageSize: number = 25, sortDesc = true): Promise<IStreamsResponse> {
    const query = qs.stringify(
        {
            populate: {
                vtuber: {
                    fields: [
                        'id',
                    ]
                }
            },
            filters: {
                vtuber: {
                    id: {
                        $eq: vtuberId
                    }
                }
            },
            pagination: {
                page: page,
                pageSize: pageSize
            },
            sort: {
                date: (sortDesc) ? 'desc' : 'asc'
            }
        }
    )
    return fetch(`${strapiUrl}/api/streams?${query}`, fetchStreamsOptions)
        .then((res) => res.json())
}


// /**
//  * This returns stale data, because futureporn-historian is broken.
//  * @todo get live data from historian
//  * @see https://gitea.futureporn.net/futureporn/futureporn-historian/issues/1
//  */
// export async function getProgress(vtuberSlug: string): Promise<{ complete: number; total: number }> {
//     const query = qs.stringify({
//         filters: {
//             vtuber: {
//                 slug: {
//                     $eq: vtuberSlug
//                 }
//             }
//         }
//     })
//     const data = await fetch(`${strapiUrl}/api/streams?${query}`, fetchStreamsOptions)
//         .then((res) => res.json())
//         .then((g) => {
//             return g
//         })

//     const total = data.meta.pagination.total

//     return {
//         complete: total,
//         total: total
//     }
// }